#import "@preview/cetz:0.3.2": draw
#import "../element.typ"
#import "../ports.typ": add-port

#let draw-shape(
  id,
  tl,
  tr,
  br,
  bl,
  fill,
  stroke,
  vertical: false,
) = {
  let (x0, y0) = tl
  let (x1, y1) = br

  let w = x1 - x0
  let h = y1 - y0

  let cx = x0 + w / 2
  let cy = y0 + h / 2
  let r = calc.min(w, h) / 2

  let line = draw.line.with(stroke: stroke)
  let circle = draw.circle.with(stroke: stroke, fill: fill)

  let f = draw.group(
    name: id,
    {
      // Kreis und Tilde-Symbol
      circle((cx, cy), radius: r)
      draw.content((cx, cy), [$tilde$], anchor: "center")

      // Verbindungslinien zu den Ports
      if vertical {
        // Vertikale Quelle
        let p_start = (cx, y0) // North port
        let p_end = (cx, y1) // South port
        let circle_top = (cx, cy - r) // Top point of the circle
        let circle_bottom = (cx, cy + r) // Bottom point of the circle
        line(p_start, circle_top) // Line from North port to top of circle
        line(circle_bottom, p_end) // Line from bottom of circle to South port
      } else {
        // Horizontale Quelle
        let p_start = (x0, cy) // West port
        let p_end = (x1, cy) // East port
        let circle_left = (cx - r, cy) // Left point of the circle
        let circle_right = (cx + r, cy) // Right point of the circle
        line(p_start, circle_right) // Line from West port to left of circle
        line(circle_left, p_end) // Line from right of circle to East port
      }
    },
  )

  return (f, tl, tr, br, bl)
}

#let ac-source(
  x: none,
  y: none,
  w: none,
  h: none,
  name: none,
  name-anchor: "center",
  vertical: false,
  fill: none,
  stroke: black + 1pt,
  id: "",
  debug: (
    ports: false,
  ),
) = {
  let ports = if vertical {
    (
      north: ((id: "0"),),
      south: ((id: "1"),),
    )
  } else {
    (
      west: ((id: "0"),),
      east: ((id: "1"),),
    )
  }

  element.elmt(
    draw-shape: draw-shape.with(vertical: vertical),
    x: x,
    y: y,
    w: w,
    h: h,
    name: name,
    name-anchor: name-anchor,
    ports: ports,
    fill: fill,
    stroke: stroke,
    id: id,
    debug: debug,
  )
}
