#import "@preview/cetz:0.3.2": draw
#import "../element.typ"
#import "../ports.typ": add-port

#let draw-shape(
  id,
  tl,
  tr,
  br,
  bl,
  fill,
  stroke,
  vertical: false,
  closed: false,
) = {
  let (x0, y0) = tl
  let (x1, y1) = br

  let w = x1 - x0
  let h = y1 - y0

  let line = draw.line.with(stroke: stroke)
  let circle = draw.circle.with(stroke: stroke, fill: white) // Use white fill for visibility
  let terminal_radius = 0.05 * calc.min(w, h) // Small radius for terminals

  let cx = x0 + w / 2 // Center x
  let cy = y0 + h / 2 // Center y

  let f = draw.group(
    name: id,
    {
      if vertical {
        // Vertical Switch
        let p0 = (cx, y0) // North port
        let p1 = (cx, y1) // South port
        let terminal0 = (cx, y0 + 0.2 * h) // Upper terminal position
        let terminal1 = (cx, y1 - 0.2 * h) // Lower terminal position

        // Draw connection lines (ending at the terminals)
        line(p0, terminal0) // Line from North port to upper terminal
        line(terminal1, p1) // Line from lower terminal to South port

        // Draw terminals (circles)
        circle(terminal0, radius: terminal_radius, name: "left-connector")
        circle(terminal1, radius: terminal_radius, name: "right-connector")

        // Draw switch lever
        let lever_end
        let lever_start
        if closed {
          lever_start = "left-connector.north"
          lever_end = "right-connector.south"
        } else {
          lever_start = "left-connector.135deg"
          lever_end = (rel: (-45deg, calc.abs(y1 - y0) * 0.6), to: "left-connector.45deg")
        }
        line(lever_start, lever_end)
      } else {
        // Horizontal Switch
        let p0 = (x0, cy) // West port
        let p1 = (x1, cy) // East port
        let terminal0 = (x0 + 0.2 * w, cy) // Left terminal position
        let terminal1 = (x1 - 0.2 * w, cy) // Right terminal position

        // Draw connection lines (ending at the terminals)
        line(p0, terminal0) // Line from West port to left terminal
        line(terminal1, p1) // Line from right terminal to East port

        // Draw terminals (circles)
        circle(terminal0, radius: terminal_radius, name: "left-connector")
        circle(terminal1, radius: terminal_radius, name: "right-connector")

        // Draw switch lever
        let lever_end
        let lever_start
        if closed {
          lever_start = "left-connector.west"
          lever_end = "right-connector.east"
        } else {
          lever_start = "left-connector.225deg"
          lever_end = (rel: (45deg, calc.abs(x1 - x0) * 0.6), to: "left-connector.45deg")
        }
        line(lever_start, lever_end)
      }
    },
  )

  return (f, tl, tr, br, bl)
}

#let switch(
  x: none,
  y: none,
  w: none,
  h: none,
  name: none,
  name-anchor: "center",
  vertical: false,
  closed: false,
  fill: none,
  stroke: black + 1pt,
  id: "",
  debug: (
    ports: false,
  ),
) = {
  let ports = if vertical {
    (
      north: ((id: "0"),),
      south: ((id: "1"),),
    )
  } else {
    (
      west: ((id: "0"),),
      east: ((id: "1"),),
    )
  }

  element.elmt(
    draw-shape: draw-shape.with(
      vertical: vertical,
      closed: closed,
    ),
    x: x,
    y: y,
    w: w,
    h: h,
    name: name,
    name-anchor: name-anchor,
    ports: ports,
    fill: fill,
    stroke: stroke,
    id: id,
    debug: debug,
  )
}
